import pygame,sys,random
from practicum import findDevices
from peri import PeriBoard
from numpy import interp

pygame.init()
#constant of game
screen = pygame.display.set_mode((640,480))
screen_width = screen.get_width()
screen_height = screen.get_height()
score = 0
win_number = 0
gamestate = 0
renderboss =False
killboss = False
devs = findDevices()
if len(devs) == 0:
    print '*** No MCU board found.'
    board = None
else:
    board = PeriBoard(devs[0])
    print '*** MCU board found'
    print '*** Device manufacturer: %s' % board.getVendorName()
    print '*** Device name: %s' % board.getDeviceName()
    print 'Calibrating board.'
    raw_input('Face the board down then press ENTER...')
    minLight = board.getLight()
    print minLight 
    raw_input('Face the board up then press ENTER...')
    maxLight = board.getLight()
    print maxLight
    board.lightRange = (minLight,maxLight)

#class of object in the game
class Boss:
    def __init__(self,xpos,ypos,image):
        self.health = 10
        self.x = xpos
        self.y = ypos
        self.cooldown = 200    
        self.speed = 100
        self.last = pygame.time.get_ticks()
        self.image = image
        self.bullets = []
        self.boss_width = image.get_width()
        self.boss_height = image.get_height()
    def render(self):
        screen.blit(self.image,(self.x,self.y))
    def move(self,direction):
        if direction == "left" and self.x > 30:
            self.x -= self.speed
        elif direction == "right" and self.x < 450:
            self.x += self.speed
        elif direction == "up" and self.y > self.boss_height:
            self.y -= self.speed
        elif direction == "down" and self.y < 250:
            self.y += self.speed
    def checkcooldown(self,time):
        if time - self.last >= self.cooldown:
            self.last = time
            return True
        return False
            
class Player:
    def __init__(self,xpos,ypos,image):
        self.health = 3
        self.x = xpos
        self.y = ypos
        self.speed = 5
        self.cooldown = 300
        self.last = pygame.time.get_ticks()
        self.image = image
        self.bullets = []
        self.player_width = image.get_width()
        self.player_height = image.get_height()
    def render(self):
        screen.blit(self.image,(self.x,self.y))
    def move(self,direction):
        if direction == "left" and self.x > 0:
            self.x -= self.speed
        elif direction == "right" and self.x < screen_width-self.player_width:
            self.x += self.speed
    def checkcooldown(self,time):
        if time - self.last >= self.cooldown:
            self.last = time
            return True
        return False
    
class Enemy:
    def __init__(self,xpos,ypos,image):
        self.x = xpos
        self.y = ypos
        self.speed = 3
        self.cooldown = 1000   
        self.last = pygame.time.get_ticks()
        self.image = image
        self.delete = False
        self.bullets = []
        self.enemy_width = image.get_width()
        self.enemy_height = image.get_height()
        self.godown = False
    def render(self):
        screen.blit(self.image,(self.x,self.y))
    def checkcooldown(self,time):
        if time - self.last >= self.cooldown:
            self.last = time
            return True
        return False

class Bullet:
    def __init__(self,xpos,ypos,image):
        self.x = xpos
        self.y = ypos
        self.image = image
        self.speed = 5
        self.bullet_width = image.get_width()
        self.bullet_height = image.get_height()
    def render(self):
        screen.blit(self.image,(self.x,self.y))

    def checkcollide(self,x1,x2,y1,y2):
        if(self.x >= x1 and self.x <= x1+x2 and self.y >= y1 and self.y <= y1+y2):
            return True
        return False
    def outofscreen(self):
        if self.y < 0 or self.y > screen_height:
            return True
        return False
    def move(self,direction):
        if direction == "left" and self.x > 0:
            self.x -= self.speed
        elif direction == "right" and self.x < screen_width-self.bullet_width:
            self.x += self.speed


class Barrier:
    def __init__(self,xpos,ypos,image):
        self.bullet = None
        self.x = xpos
        self.y = ypos
        self.count = 0
        self.bosshealth = 5
        self.speed = 5
        self.image = image
        self.barrier_width = image.get_width()
        self.barrier_height = image.get_height()
    def set_position(self,xpos,ypos):
        self.x = xpos
        self.y = ypos
    def render(self):
        screen.blit(self.image,(self.x,self.y))        
    def move(self,direction):
        if direction == "left" and self.x > 0:
            self.x -= self.speed
        elif direction == "right" and self.x < screen_width-self.barrier_width:
            self.x += self.speed

def Titlescreen():
    exit = False
    global gamestate
    global killboss
    global renderboss
    global win_number
    killboss = False
    renderboss =False
    win_number = 0
    player =  Player(304,448,pygame.image.load("picture/spaceship.bmp").convert_alpha())
    enemy =  Enemy(304,0,pygame.image.load("picture/enemy1.bmp").convert_alpha())
    image = pygame.image.load("picture/TITLE.bmp").convert_alpha()
    clock = pygame.time.Clock()    
    while not exit:
        pygame.init()
        screen.fill((0,0,0))          
        screen.blit(image,(0,0))
        clock.tick(60)
        player.render()
        enemy.render()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
        if board.getSwitch1() == True and len(player.bullets) < 1:
            player.bullets.append(Bullet(player.x+player.player_width/2,player.y,pygame.image.load("picture/bullet.bmp").convert_alpha()))    
        
        if len(player.bullets) == 1:
            player.bullets[0].y -= 5
            player.bullets[0].render()
            if player.bullets[0].checkcollide(enemy.x,enemy.enemy_width,enemy.y,enemy.enemy_height):
                enemy.image = pygame.image.load("picture/edeath.bmp").convert_alpha()
                del player.bullets[0]
                gamestate = 1
                exit = True
                                
        pygame.display.flip()                            
        
def Gameover():
    pygame.time.delay(200)    
    startgame = False
    global gamestate
    global killboss
    global renderboss
    killboss = False
    renderboss =False
    win_number = 0
    image = pygame.image.load("picture/gameover.bmp").convert_alpha()
    clock = pygame.time.Clock()    
    while not startgame:
        pygame.init()
        screen.fill((0,0,0))          
        screen.blit(image,(0,0))
        clock.tick(60)   
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
        if board.getSwitch1():
            startgame = True
            gamestate = 1
        pygame.display.update()              
  

def Pause():
    pygame.time.delay(200)
    image = pygame.image.load("picture/pause.bmp").convert_alpha()
    startgame = False
    global gamestate
    clock = pygame.time.Clock()    
    while not startgame:
        pygame.init()
        screen.blit(image,(0,0))
        clock.tick(60)        
        #Continue
        if board.getSwitch1():
            startgame = True
        if board.getSwitch2() and board.getSwitch3():
            startgame = True
            gamestate = 0
        pygame.display.update()      

def Win():
    pygame.time.delay(200)
    image = pygame.image.load("picture/bosskill.bmp").convert_alpha()
    startgame = False
    global gamestate
    clock = pygame.time.Clock()    
    while not startgame:
        pygame.init()
        screen.blit(image,(0,0))
        clock.tick(60)        
        if board.getSwitch1():
            startgame = True
            gamestate = 1
        pygame.display.update()      



def main():
    board.setLedValue(3)
    #Game picture
    pygame.init()
    clock = pygame.time.Clock()
    enemies = []
    x = 0
    y = 0
    global win_number
    global gamestate
    global renderboss
    global killboss
    global score
    #add enemy
    boss = None
    numberofenemy = 0
    if win_number <= 3:
        if not killboss:
            for column in range(win_number + 1):
                for count in range(10):
                    enemies.append(Enemy(50*x + 50,50*y + 50,pygame.image.load("picture/enemy1.bmp").convert_alpha()))
                    numberofenemy += 1
                    x += 1
                x = 0
                y += 1
        else:
            for column in range(3):
                for count in range(10):
                    enemies.append(Enemy(50*x + 50,50*y + 50,pygame.image.load("picture/enemy1.bmp").convert_alpha()))
                    x += 1
                x = 0
                y += 1


    #create new object
    player =  Player(304,448,pygame.image.load("picture/spaceship.bmp").convert_alpha())
    barrier = Barrier(304,428,pygame.image.load("picture/barrier.bmp").convert_alpha())
    boss = Boss(304,50,pygame.image.load("picture/boss.bmp").convert_alpha())
    barrierbullet = Bullet(304,-100,pygame.image.load("picture/ultimate.bmp").convert_alpha())
    
    exit = False

    #Gamestart here
    while not exit:
        clock.tick(60)
        screen.fill((0,0,0))
        #create enemy        

        #enemy action
        for count in range(len(enemies)):
            now = pygame.time.get_ticks()
            enemies[count].render()
            enemies[count].x += enemies[count].speed
            #Game over
            if enemies[count].y >= player.y-32:
                exit = True
                gamestate =3
            #enemy move right
            if enemies[count].x > screen_width-enemies[0].enemy_width: 
                for count1 in range(len(enemies)):
                    enemies[count1].y += 5
                    enemies[count1].speed = -3
                    enemies[count1].image = pygame.image.load("picture/enemy1.bmp")
                break
            
            #enemy move left
            if enemies[count].x < 0:
                for count1 in range(len(enemies)):
                    enemies[count1].y += 5
                    enemies[count1].speed = 3
                    enemies[count1].image = pygame.image.load("picture/enemy2.bmp").convert_alpha()
                break
            
            #random if enemy have to shoot or not
            rand = random.randint(0,100)            
            if rand < 5 and enemies[count].checkcooldown(now):
                if len(enemies[count].bullets) < 1:
                    enemies[count].bullets.append(Bullet(enemies[count].x+enemies[0].enemy_width/2,\
                    enemies[count].y,pygame.image.load("picture/enemybullet.bmp").convert_alpha()))
            
            #enemy bullet move
            if len(enemies[count].bullets) > 0:
                enemies[count].bullets[0].y += 3
                
            #Check if enemy bullet hit barrier    
            if len(enemies[count].bullets) > 0:                
                if enemies[count].bullets[0].checkcollide(barrier.x,barrier.barrier_width,barrier.y,barrier.barrier_height):
                    barrier.count += 1
                    del enemies[count].bullets[0]
         
        
            #Check if enemy bullet hit player
            if len(enemies[count].bullets) > 0:
                if enemies[count].bullets[0].checkcollide(player.x,player.player_width,player.y,player.player_height):
                    player.health -= 1
                    del enemies[count].bullets[0]
                    if player.health <= 0:
                            board.setLedValue(0)
                            exit = True
                            win_number = 0
                            gamestate = 3
     
            #Check if player bullet hit enemy
            if len(player.bullets)>0:
                if player.bullets[0].checkcollide(enemies[count].x,enemies[0].enemy_width,enemies[count].y,enemies[0].enemy_height):
                        enemies[count].image = pygame.image.load("picture/edeath.bmp").convert_alpha()                   
                        enemies[count].delete = True
                        del player.bullets[0]
                        break
            if enemies[count].delete:
                pygame.display.flip()
                pygame.time.delay(5)
                del enemies[count]
                break


            
            #Check if barrier bullet hit enemy
            if barrierbullet.checkcollide(enemies[count].x,enemies[0].enemy_width,enemies[count].y,enemies[0].enemy_height):
                del enemies[count]
                break
           
            #Check enemy bullet outofscreen
            if len(enemies[count].bullets) > 0:
                if enemies[count].bullets[0].outofscreen():
                    del enemies[count].bullets[0]    
           
            #render enemy bullet
            if len(enemies[count].bullets) > 0:
                enemies[count].bullets[0].render()
        
###########################################################################################

        #player bullet move
        for count in range(len(player.bullets)):
             if player.bullets[count].y <screen_height and player.bullets[count].y > 0:
                player.bullets[count].render()
                player.bullets[count].y -= 5
        
        #barrier bullet move
        if barrierbullet.y <screen_height and barrierbullet.y > 0:
            barrierbullet.render()
            barrierbullet.y -= 2
                                
        #Check player bullet out of screen
        if len(player.bullets)>0:
            if player.bullets[0].outofscreen():
                del player.bullets[0]
        
        #Check barrier bullet out of screen
        if barrierbullet.outofscreen():
            barrierbullet.y = -100
                


        #Check number of enemy if none You win
        if len(enemies) == 0 and not renderboss:
            win_number += 1
            if win_number < 3:
                exit = True
                score += 1000
                gamestate = 1
            else:
                exit = True
                gamestate = 1
                win_number += 1
                renderboss = True

        #Boss Action
        if win_number >= 4:
            now = pygame.time.get_ticks()
            randbossaction = random.randint(0,100)     
            #Check if missile hit boss
            if randbossaction < 40 :
                if randbossaction < 10 and boss.checkcooldown(now):
                    boss.move("up")
                elif randbossaction < 20 and boss.checkcooldown(now):
                    boss.move("down")
                elif randbossaction < 30 and boss.checkcooldown(now):
                    boss.move("left")
                elif randbossaction < 40 and boss.checkcooldown(now):
                    boss.move("right")
            if randbossaction <= 100:
                if boss.checkcooldown(now) and len(boss.bullets) < 30:
                    boss.bullets.append(Bullet(random.randint(32,448),boss.y,pygame.image.load("picture/bossbullet.bmp").convert_alpha()))

            for count in range(len(boss.bullets)):
                 if boss.bullets[count].y <screen_width and boss.bullets[count].y > 0:
                    boss.bullets[count].render()
                    boss.bullets[count].y += 10           

            if len(player.bullets)>0:
                if player.bullets[0].checkcollide(boss.x,boss.boss_width,boss.y,boss.boss_height):
                    boss.health -= 1
                    del player.bullets[0]


            if barrierbullet.checkcollide(boss.x,boss.boss_width,boss.y,boss.boss_height):
                boss.health -= 15
                barrierbullet.y = -100


                    
            if len(boss.bullets) > 0:
                if boss.bullets[0].checkcollide(barrier.x,barrier.barrier_width,barrier.y,barrier.barrier_height):
                    barrier.count += 1
                    del boss.bullets[0]
    
                elif boss.bullets[0].checkcollide(player.x,player.player_width,player.y,player.player_height):
                    player.health -= 1
                    del boss.bullets[0]
                    if player.health <= 0:
                        exit = True
                        win_number = 0
                        gamestate = 3
                elif boss.bullets[0].outofscreen():
                    del boss.bullets[0]                        

        #Control Part
        #Player move  
        if board.getSwitch2()==True:
            player.move("right")
        if board.getSwitch3()==True:
            player.move("left")
        #Barrier move
        if board.getLight()>(maxLight+minLight)/2 and barrier.x < screen_width - barrier.barrier_width:
            barrier.move("right")            
        if board.getLight()<=(maxLight+minLight)/2 and barrier.x>0:
            barrier.move("left")
        #Barrier bullet move
        if board.getLight()>(maxLight+minLight)/2 and barrierbullet.x < screen_width - barrierbullet.bullet_width:
            barrierbullet.move("right")
        if board.getLight()<=(maxLight+minLight)/2 and barrierbullet.x>0:
            barrierbullet.move("left")
        #SHOOT
        if board.getSwitch1()==True:
            now = pygame.time.get_ticks()
            if(len(player.bullets)!=5) and player.checkcooldown(now):
                player.bullets.append(Bullet(player.x+player.player_width/2,player.y,pygame.image.load("picture/bullet.bmp").convert_alpha()))                
        #Pause screen
        if board.getSwitch1() and board.getSwitch2() and board.getSwitch3():
            Pause()
            if gamestate != 1:
                exit = True

        #Display player health on LED
        if player.health == 3:
            board.setLedValue(3)
        elif player.health == 2:
            board.setLedValue(2)
            player.image = pygame.image.load("picture/spaceship1.bmp").convert_alpha()
        elif player.health == 1:
            board.setLedValue(1)
            player.image = pygame.image.load("picture/spaceship2.bmp").convert_alpha()


        if barrier.count == 0:
            barrier.image = pygame.image.load("picture/barrier.bmp").convert_alpha()
        elif barrier.count == 1:
            barrier.image = pygame.image.load("picture/barrier1.bmp").convert_alpha()
        elif barrier.count == 2:
            barrier.image = pygame.image.load("picture/barrier2.bmp").convert_alpha()
        elif barrier.count == 3:
            barrier.image = pygame.image.load("picture/barrier3.bmp").convert_alpha()
        elif barrier.count >= 4:
            barrier.image = pygame.image.load("picture/barrier4.bmp").convert_alpha()
            board.setLedValue(4) 


        #Render object   
        #Render player
        player.render()
        #Render barrier
        barrier.render()
  
        #Render barrier bullet
        if barrier.count>=4 and board.getSwitch2() and board.getSwitch3():
            barrierbullet.x = barrier.x+barrier.barrier_width/2
            barrierbullet.y = barrier.y - 10
            barrier.count = 0
            board.setLedValue(5)
            
        #Render boss
        if win_number == 4 :
            if boss.health>0:
                boss.render()    
            if boss.health<=0:
                killboss = True
                renderboss = False
                win_number = 0
                gamestate = 4
                exit = True
        pygame.display.flip()
              
        
        
if __name__ == "__main__":
    while True:
        if gamestate == 0:
            Titlescreen()
        elif gamestate == 1:
            main()
        elif gamestate == 2:
            Pause()
        elif gamestate == 3:
            Gameover()
        elif gamestate == 4:
            Win()
    
            