from practicum import McuBoard

RQ_SET_LED    = 0
RQ_GET_LIGHT  = 2
RQ_GET_SWITCH1 = 3
RQ_GET_SWITCH2 = 4
RQ_GET_SWITCH3 = 5

####################################
class PeriBoard(McuBoard):

    ################################
    def setLed(self, led_no, led_val):
        '''
        Set status of LED led_no on peripheral board to led_val
        '''
        self.usbWrite(request=RQ_SET_LED,index=led_no,value=led_val)

    ################################
    def setLedValue(self, value):
        '''
        Display value's 3 LSBs on peripheral board's LEDs
        '''
        if value == 5:
            self.usbWrite(request=RQ_SET_LED,index=0,value=0)
        if value == 4:
            self.usbWrite(request=RQ_SET_LED,index=0,value=1)
        if value == 3:
            self.usbWrite(request=RQ_SET_LED,index=0,value=0)
            self.usbWrite(request=RQ_SET_LED,index=1,value=1)
            self.usbWrite(request=RQ_SET_LED,index=2,value=1)
            self.usbWrite(request=RQ_SET_LED,index=3,value=1)
        if value == 2:
            self.usbWrite(request=RQ_SET_LED,index=0,value=0)
            self.usbWrite(request=RQ_SET_LED,index=1,value=1)
            self.usbWrite(request=RQ_SET_LED,index=2,value=1)
            self.usbWrite(request=RQ_SET_LED,index=3,value=0)
        if value == 1:
            self.usbWrite(request=RQ_SET_LED,index=0,value=0)
            self.usbWrite(request=RQ_SET_LED,index=1,value=1)
            self.usbWrite(request=RQ_SET_LED,index=2,value=0)
            self.usbWrite(request=RQ_SET_LED,index=3,value=0)
        if value == 0:
            self.usbWrite(request=RQ_SET_LED,index=0,value=0)
            self.usbWrite(request=RQ_SET_LED,index=1,value=0)
            self.usbWrite(request=RQ_SET_LED,index=2,value=0)
            self.usbWrite(request=RQ_SET_LED,index=3,value=0)

        

    ################################
    def getSwitch1(self):
        '''
        Return a boolean value indicating whether the switch on the peripheral
        board is currently pressed
        '''
        x = self.usbRead(request = RQ_GET_SWITCH1,length=1)
        if x[0] == 1:
            return True
        else:
            return False
    ################################
    def getSwitch2(self):
        '''
        Return a boolean value indicating whether the switch on the peripheral
        board is currently pressed
        '''
        x = self.usbRead(request = RQ_GET_SWITCH2,length=1)
        if x[0] == 1:
            return True
        else:
            return False
    ################################
    def getSwitch3(self):
        '''
        Return a boolean value indicating whether the switch on the peripheral
        board is currently pressed
        '''
        x = self.usbRead(request = RQ_GET_SWITCH3,length=1)
        if x[0] == 1:
            return True
        else:
            return False
    ################################
    def getLight(self):
        '''
        Return the current reading of light sensor on peripheral board
        '''
        x = self.usbRead(request = RQ_GET_LIGHT,length=2)
        return x[1]*256+x[0]
