#include <usbdrv.h>

#define RQ_SET_LED    0
#define RQ_GET_LIGHT  2
#define RQ_GET_SWITCH1 3
#define RQ_GET_SWITCH2 4
#define RQ_GET_SWITCH3 5
//////////////////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
  usbRequest_t *rq = (usbRequest_t*)data;
  static uint8_t switch_state;
  static uint16_t light_val;

  if (rq->bRequest == RQ_SET_LED)
  {
    uint8_t led_val = rq->wValue.bytes[0];
    uint8_t led_no  = rq->wIndex.bytes[0];

    if (led_no == 0)
      digitalWrite(PIN_PC0, led_val);
    else if (led_no == 1)
      digitalWrite(PIN_PC1, led_val);
    else if (led_no == 2)
      digitalWrite(PIN_PC2, led_val);
    else if (led_no == 3)
      digitalWrite(PIN_PC3, led_val);

    return 0;  // return no data back to host
  }

  else if (rq->bRequest == RQ_GET_SWITCH1)
  {
    if (digitalRead(PIN_PD0) == LOW) /* switch is pressed */ 
      switch_state = 1;
    else
      switch_state = 0;

    /* point usbMsgPtr to the data to be returned to host */
    usbMsgPtr = (uint8_t*) &switch_state;

    /* return the number of bytes of data to be returned to host */
    return sizeof(switch_state);
  }
  
  else if (rq->bRequest == RQ_GET_SWITCH2)
  {
    if (digitalRead(PIN_PD1) == LOW) /* switch is pressed */ 
      switch_state = 1;
    else
      switch_state = 0;

    /* point usbMsgPtr to the data to be returned to host */
    usbMsgPtr = (uint8_t*) &switch_state;

    /* return the number of bytes of data to be returned to host */
    return sizeof(switch_state);
  }
  
  else if (rq->bRequest == RQ_GET_SWITCH3)
  {
    if (digitalRead(PIN_PD5) == LOW) /* switch is pressed */ 
      switch_state = 1;
    else
      switch_state = 0;

    /* point usbMsgPtr to the data to be returned to host */
    usbMsgPtr = (uint8_t*) &switch_state;

    /* return the number of bytes of data to be returned to host */
    return sizeof(switch_state);
  }

  else if(rq->bRequest == RQ_GET_LIGHT)
  {
    light_val = analogRead(PIN_PC4);
    usbMsgPtr = (uint8_t*) &light_val;
    return 2;
  }

  return 0;   /* nothing to do; return no data back to host */
}

//////////////////////////////////////////////////////////////////////
void setup()
{
    pinMode(PIN_PC0,OUTPUT);
    pinMode(PIN_PC1,OUTPUT);
    pinMode(PIN_PC2,OUTPUT);
    pinMode(PIN_PC3,OUTPUT);
    pinMode(PIN_PC4,INPUT);
    pinMode(PIN_PD3,OUTPUT);
    
    pinMode(PIN_PD0,INPUT_PULLUP);
    pinMode(PIN_PD1,INPUT_PULLUP);
    pinMode(PIN_PD5,INPUT_PULLUP);

    usbInit();

    /* enforce re-enumeration of USB devices */
    usbDeviceDisconnect();
    delay(300);
    usbDeviceConnect();
}

//////////////////////////////////////////////////////////////////////
void loop()
{
  usbPoll();
}
